#from naw.nawinternal import verify_connection
import naw.nawinternal
from naw.nawinternal import (INIT_CLI, INIT_SRV, CONFIGDIR)
from naw.nawinternal import (verify_connection, verify_connection_ex)

import socket as sockmod
from socket import *


sockclass = sockmod._socketobject

class nawify(sockclass):
    """nawify(sd)

    Create a naw wrapper around a socket.

    The resulting object should be used instead of the actual socket
    for all feasible operations.  Usage is identical, except for the
    addition of methods verify() and verify_ex().
    """

    def __init__(self, socketobj):
        self.sd = socketobj
        self._sock = socketobj

    def connect(self, arg):
        """Connect a socket and (if appropriate) verify the connection.

        Usage is identical to [socket].connect(...).
        """
        sockclass.connect(self.sd, arg)
        self.verify(INIT_CLI)

    def accept(self):
        """Accept a socket connection and (if appropriate) verify it.

        Usage is identical to [socket].accept(...).
        """
        sockclass.accept(self.sd)
        self.verify(INIT_SRV)

    def connect_ex(self, arg):
        """Connect a socket and (if appropriate) verify the connection.
        Errno will be returned on error; 0 otherwise.

        Usage is identical to [socket].connect_ex(...).
        """
        errno = sockclass.connect_ex(self.sd, arg)
        if errno:
            return errno

        return self.verify_ex(INIT_CLI)

    def verify(self, svc_type):
        """Verify a currently connected socket.
        On error, the descriptor will be closed and an exception,
        with errno carryover from naw_verify(), will be raised.
        """
        sd = self.fileno()
        err_type = sockmod.error
        verify_connection(err_type, sd, svc_type)

    def verify_ex(self, svc_type):
        """Verify a currently connected socket without an exception.
        On error, the descriptor will be closed and errno, as carried
        over from naw_verify(), will be returned.
        """
        sd = self.fileno()
        err_type = sockmod.error
        return verify_connection_ex(err_type, sd, svc_type)

    def close(self):
        """Close the underlying socket.

        Identical to [socket].close().
        """
        self.sd.close()

def socket(type):
    """Create a new, nawified socket.

    Usage is identical to socket.socket().
    """
    return nawify(sockmod.socket(type))

conf = naw.nawinternal.conf()
