#include <sys/types.h>
#include <sys/mman.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include "lxmisc.h"
#include "misc.h"
#include "naw_log.h"

static ub4_t *module_flush_counter;
static ub4_t module_flush_counter_last;

/**************************************************************************
  module_check_flush

  returns 1 if the module counter has not changed
  returns 0 if it has, and the modules should be flushed.
 **************************************************************************/

int module_check_flush (void) {
    return (*module_flush_counter == module_flush_counter_last);
}

/**************************************************************************
  module_reset_flush_counter

  reset the flush counter
 **************************************************************************/

void module_reset_flush_counter (void) {
    module_flush_counter_last = *module_flush_counter;
}

/**************************************************************************
  setup_flushfile

  opens the flush file, maps it into shared memory, closes the descriptor,
  and continues life as normal.  module_flush_counter will point into the
  four bytes of shared memory necessary for this process.

  Note that module_flushfile _must_ contain at least four bytes of data, or
  this operation will result in a bus error.

  If module_flushfile does not exist, we continue on as normal -- but
  without the option of flushing modules.
 **************************************************************************/

void setup_flushfile (void) {
    int fd;
    void *flushaddr;


    fd = open (module_flushfile, O_RDONLY);

    if (fd == -1) {
        naw_logfmt(NAW_INFO, "open(%s): %s", module_flushfile,
                strerror(errno));
        module_flush_counter_last = 0;
        module_flush_counter = &module_flush_counter_last;
        return;
    }

    flushaddr = mmap(0, 4, PROT_READ, MAP_SHARED, fd, 0);

    if (flushaddr == (void *)-1)
        die_open(module_flushfile);

    close (fd);

    module_flush_counter = (ub4_t *)flushaddr;
    module_flush_counter_last = *module_flush_counter;
}
