#include <dlfcn.h>
#include <stdlib.h>
#include <string.h>
#include <minimisc.h>
#include "lxmisc.h"
#include "misc.h"
#include "naw_log.h"

#define LXM_s(s) LXM_mb(s, strlen(s))

struct naw_module *naw_module_new (name, data)
    ub1_t *name;
    struct naw_data *data;
{
    struct naw_module *entry = malloc (sizeof(struct naw_module));
    memset (entry, 0, sizeof(struct naw_module));

    NAW_LOG(NAW_DEBUG3,
            LXM_q("loading new module ("), LXM_s(name), LXM_q(")"));

    if (!(entry->handle = dlopen (name, RTLD_LAZY))) {
        naw_logfmt(NAW_NOTICE,
                "dlopen(%s): %s", name, (char *)dlerror());
        free(entry);
        return 0;
    }

    if (!(entry->f = dlsym (entry->handle, "naw_set"))) {
        naw_logfmt(NAW_NOTICE,
                "dlsym(%s:naw_set): %s", name, (char *)dlerror());
        dlclose(entry->handle);
        free(entry);
        return 0;
    }

    if (entry->f->init) {
        if (entry->f->init(data)) { /* != NAW_OK */
            NAW_LOG(NAW_NOTICE,
                    LXM_q("initialization failed ("),
                    LXM_s(name),
                    LXM_q(")"));
            dlclose(entry->handle);
            free(entry);
            return 0;
        }
    }

    return entry;
}

void naw_module_free(entry)
    void *entry;
{
    struct naw_module *e = (struct naw_module *)entry;

    if (e->f->fini)
        e->f->fini();

    dlclose(e->handle);
    free (entry);
}
