#include "auto-stage.h"

#include <sys/types.h>

/* get around sys/socket.h declaration of connect() */
#ifdef NAW_PRELOAD
#define connect inc_connect_proto
#endif
#include <sys/socket.h>
#undef connect

#include <errno.h>
#include <string.h>
//#include <netinet/in.h> /* me */
#include "lxmisc.h"
#include "naw_log.h"
#include "misc.h"

#ifdef NAW_PRELOAD
static int (*realconnect)(int, const struct sockaddr *, socklen_t);

void wrap_connect (void) {
    realconnect = get_funcptr ("connect");
}
#else
# define realconnect connect
#endif

int naw_connect (sockfd, serv_addr, addrlen)
    int sockfd;
    const struct sockaddr *serv_addr;
    socklen_t addrlen;
{
    int r, socktype;

    EXPLICIT_INIT(if (!naw_initialized) naw_init());

    NAW_LOG (NAW_DEBUG2, LXM_q("connect() was called."));

    r = sizeof(socktype);
    if (getsockopt (sockfd, SOL_SOCKET, SO_TYPE, &socktype, &r))
        return (-1);

    r = realconnect (sockfd, serv_addr, addrlen);
    if (r == -1) {
        naw_logfmt (NAW_DEBUG, "connect failed: %s",strerror(errno));
        return (r);
    }

    if (socktype != SOCK_STREAM) {
        NAW_LOG (NAW_DEBUG3,
                LXM_q("connect() used non-stream socket (ignoring)"));
        return (r);
    }

    r = naw_verify_internal (sockfd, INIT_CLI, (void *)serv_addr);
    if (r == -1) return (r);
    else         return (0);
}

